//-----------------------------------------------------------------------------
// File: SoundFX.cpp
//
// Desc: Sample to demonstrate soundFX and parameters in directX
//
// Copyright ( c ) 1998-2001 Microsoft Corporation. All rights reserved.
//-----------------------------------------------------------------------------
(*----------------------------------------------------------------------------*
 *  DirectSound sample from DirectX 9.0 SDK                                   *
 *  Delphi adaptation by Alexey Barkovoy (e-mail: clootie@reactor.ru)         *
 *                                                                            *
 *  Delphi versions 5-7 are supported                                         *
 *                                                                            *
 *  Modified: 11-Sep-2003                                                     *
 *                                                                            *
 *  Latest version can be downloaded from:                                    *
 *     http://clootie.narod.ru/delphi                                         *
 *----------------------------------------------------------------------------*)

unit SoundFXUnit;

{$I DirectX.inc}

interface

uses
  Windows, Messages, CommCtrl,
  DirectSound, DXErr9, DSUtil, DXUtil,
  Resource;

// C++Builder compile hack
{$NOINCLUDE Resource}
{$HPPEMIT '#include "Resource.hpp"'}

const
  GUID_NULL : TGUID = '{00000000-0000-0000-0000-000000000000}';
const // from 'mmreg.h'
  WAVE_FORMAT_IEEE_FLOAT                 = $0003; (* Microsoft Corporation *)

//-----------------------------------------------------------------------------
// Function-prototypes
//-----------------------------------------------------------------------------
function MainDlgProc(hDlg: HWND; msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;

procedure OnInitDialog(hDlg: HWND);
procedure OnTimer(hDlg: HWND);
procedure OnOpenSoundFile(hDlg: HWND);
function OnPlaySound(hDlg: HWND): HRESULT;
procedure OnEffectChanged(hwndDlg: HWND);
function ValidateWaveFile(hDlg: HWND; strFileName: PChar): HRESULT;
// function CreateAndFillBuffer(hDlg: HWND; dwCreationFlags: DWORD): HRESULT; -- never used !!!
//procedure SetBufferOptions(lFrequency, lPakn, lVolume: Longint);
procedure EnablePlayUI(hDlg: HWND; bEnable: BOOL);

procedure LoadParameterUI(hwndDlg: HWND; dwFXType: DWORD);
procedure ResetParameterUI(hwndDlg: HWND);



//-----------------------------------------------------------------------------
// Name: enum ESFXType
// Desc: each is a unique identifier mapped to a DirectSoundFX
//-----------------------------------------------------------------------------
type
  TESFXType = (
    eSFX_chorus {= 0},
    eSFX_compressor,
    eSFX_distortion,
    eSFX_echo,
    eSFX_flanger,
    eSFX_gargle,
    eSFX_parameq,
    eSFX_reverb
   );

const
  eNUM_SFX = Word(High(TESFXType))+1;


//-----------------------------------------------------------------------------
// Name: class CSoundFXManager
// Desc: Takes care of effects for one DirectSoundBuffer
//-----------------------------------------------------------------------------
type
  CSoundFXManager = class
  public
    constructor Create;
    destructor Destroy; override;

  public // interface
    function Initialize(lpDSB: IDirectSoundBuffer; bLoadDefaultParamValues: BOOL): HRESULT;
    function UnInitialize: HRESULT;

    function SetFXEnable(esfxType: DWORD): HRESULT;
    function ActivateFX: HRESULT;
    function DisableAllFX: HRESULT;
    function LoadCurrentFXParameters: HRESULT;

  public // members
    m_lpChorus:      IDirectSoundFXChorus8;
    m_lpCompressor:  IDirectSoundFXCompressor8;
    m_lpDistortion:  IDirectSoundFXDistortion8;
    m_lpEcho:        IDirectSoundFXEcho8;
    m_lpFlanger:     IDirectSoundFXFlanger8;
    m_lpGargle:      IDirectSoundFXGargle8;
    m_lpParamEq:     IDirectSoundFXParamEq8;
    m_lpReverb:      IDirectSoundFXWavesReverb8;

    m_paramsChorus:      TDSFXChorus;
    m_paramsCompressor:  TDSFXCompressor;
    m_paramsDistortion:  TDSFXDistortion;
    m_paramsEcho:        TDSFXEcho;
    m_paramsFlanger:     TDSFXFlanger;
    m_paramsGargle:      TDSFXGargle;
    m_paramsParamEq:     TDSFXParamEq;
    m_paramsReverb:      TDSFXWavesReverb;

    m_lpDSB8:        IDirectSoundBuffer8;

  protected
    m_rgFxDesc: array[0..eNUM_SFX-1] of TDSEffectDesc;
    m_rgRefGuids: array[0..eNUM_SFX-1] of PGUID; // of constant pointers
    m_rgPtrs: array[0..eNUM_SFX-1] of PPointer;
    m_rgLoaded: array[0..eNUM_SFX-1] of BOOL;
    m_dwNumFX: DWORD;

    function EnableGenericFX(const guidSFXClass: TGUID; const rguidInterface: TGUID; ppObj: PPointer): HRESULT;
    function LoadDefaultParamValues: HRESULT;
  end;




//-----------------------------------------------------------------------------
// defines and global variables
//-----------------------------------------------------------------------------
const
  DEFAULT_SLIDER_MIN          = 1;
  DEFAULT_SLIDER_MAX          = $7FFFFF;
  DEFAULT_SLIDER_INC          = DEFAULT_SLIDER_MAX shr 11;

var
  g_lpSoundManager: CSoundManager = nil;
  g_lpSound: CSound = nil;
  g_lpFXManager: CSoundFXManager = nil;
  g_hInst: THandle{HINSTANCE} = 0;
  g_tszFilename: array[0..MAX_PATH-1] of Char;
  g_dwCurrentFXType: TESFXType {DWORD} = eSFX_chorus;

const
  g_tszFXNames: array[0..7] of PChar = (
    'Chorus', 'Compressor', 'Distortion', 'Echo',
    'Flanger', 'Gargle', 'Param Eq', 'Reverb'
   );



implementation

uses SysUtils, MMSystem, CommDlg;

const
  UnitName = 'SoundFXUnit.pas';


//-----------------------------------------------------------------------------
// Name: MainDlgProc()
// Desc: Handles dialog messages
//-----------------------------------------------------------------------------
function MainDlgProc(hDlg: HWND; msg: UINT; wParam: WPARAM; lParam: LPARAM): LRESULT; stdcall;
begin
  case msg of
    WM_COMMAND:
    begin
      case LOWORD(wParam) of
        IDOK, IDCANCEL: EndDialog(hDlg, IDOK);
        IDC_BUTTON_OPEN: OnOpenSoundFile(hDlg);

        IDC_BUTTON_PLAY:
        begin
          Result:= OnPlaySound(hDlg);
          if FAILED(Result) then
          begin
            // DXTRACE_ERR_MSGBOX('OnPlaySound', Result);
            MessageBox(hDlg, 'Error playing DirectSound buffer.'+
                       'Sample will now exit.', 'DirectSound Sample',
                       MB_OK or MB_ICONERROR);
            EndDialog(hDlg, IDABORT);
          end;
        end;

        IDC_BUTTON_STOP:
        begin
          if (g_lpSound <> nil) then
          begin
            g_lpSound.Stop;
            g_lpSound.Reset;
          end;
          g_lpFXManager.DisableAllFX;
          EnablePlayUI(hDlg, True);
          SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Sound stopped.');
        end;

        IDC_RADIO_TRIANGLE,
        IDC_RADIO_SQUARE,
        IDC_RADIO_SINE,
        IDC_RADIO_NEG_180,
        IDC_RADIO_NEG_90,
        IDC_RADIO_ZERO,
        IDC_RADIO_90,
        IDC_RADIO_180:
          OnEffectChanged(hDlg);

        IDHELP: DXUtil_LaunchReadme(hDlg);

        else
          if (LOWORD(wParam) >= IDC_RADIO_CHORUS) and
             (LOWORD(wParam) <= IDC_RADIO_REVERB) then
          begin
            g_dwCurrentFXType := TESFXType(LOWORD(wParam) - IDC_RADIO_CHORUS);
            LoadParameterUI(hDlg, DWORD(g_dwCurrentFXType));
          end else
          begin
            Result := 0{FALSE}; // Didn't handle message
            Exit;
          end;
      end;
    end;

    WM_TIMER: OnTimer(hDlg);

    WM_INITDIALOG: OnInitDialog(hDlg);

    WM_NOTIFY:
    begin
      //LPNMHDR pnmh = ( LPNMHDR ) lParam;
      //if( pnmh->code >= IDC_SLIDER1 && pnmh->code <= IDC_SLIDER6 )
      //{
        OnEffectChanged(hDlg);
      //}
    end;

    WM_DESTROY:
    begin
      // Cleanup everything
      KillTimer(hDlg, 1);
      SAFE_DELETE(g_lpFXManager);
      SAFE_DELETE(g_lpSound);
      SAFE_DELETE(g_lpSoundManager);
    end;

    else
      Result := 0{FALSE}; // Didn't handle message
      Exit;
  end;

  Result:= 1{TRUE}; // Handled message
end;


//-----------------------------------------------------------------------------
// Name: OnInitDialog()
// Desc: Initializes the dialogs (sets up UI controls, etc.)
//-----------------------------------------------------------------------------
procedure OnInitDialog(hDlg: HWND);
var
  hr: HRESULT;
  hIcon_: HICON;
  strFile: array[0..MAX_PATH-1] of Char;
begin
  // Load the icon - will be set during WM_INITDIALOG
  hIcon_ := LoadIcon(g_hInst, MAKEINTRESOURCE(IDI_ICON));
  PostMessage(hDlg, WM_SETICON, ICON_BIG,  LPARAM(hIcon_));
  PostMessage(hDlg, WM_SETICON, ICON_SMALL, LPARAM(hIcon_));

  // Create a static IDirectSound in the CSound class.
  // Set coop level to DSSCL_PRIORITY, and set primary buffer
  // format to stereo, 22kHz and 16-bit output.
{$IFDEF SUPPORTS_EXCEPTIONS}
  try
    g_lpSoundManager := CSoundManager.Create;
    g_lpFXManager := CSoundFXManager.Create;
  except
    on EOutOfMemory do
    begin
      // DXTrace(__FILE__, __LINE__, Result, 'Initialize', True);
      DXTrace(UnitName, $FFFFFFFF, E_OUTOFMEMORY, 'Initialize', True);
      EndDialog(hDlg, IDABORT);
      Exit;
    end;
    else raise; 
  end;
{$ELSE}
  g_lpSoundManager := CSoundManager.Create;
  g_lpFXManager := CSoundFXManager.Create;

  if (nil = g_lpSoundManager) or
     (nil = g_lpFXManager) then
  begin
    // DXTrace(__FILE__, __LINE__, Result, 'Initialize', True);
    DXTrace(UnitName, $FFFFFFFF, E_OUTOFMEMORY, 'Initialize', True);
    EndDialog(hDlg, IDABORT);
    Exit;
  end;
{$ENDIF}


  hr := g_lpSoundManager.Initialize(hDlg, DSSCL_PRIORITY);
  if FAILED(hr) then 
  begin
    // DXTrace(__FILE__, __LINE__, hr, 'Initialize', True);
    DXTrace(UnitName, $FFFFFFFF, hr, 'Initialize', True);
    MessageBox(hDlg, 'Error initializing DirectSound.  Sample will now exit.',
                     'DirectSound Sample', MB_OK or MB_ICONERROR);
    EndDialog(hDlg, IDABORT);
    Exit;
  end;

  hr := g_lpSoundManager.SetPrimaryBufferFormat(2, 22050, 16);
  if FAILED(hr) then
  begin
    // DXTrace(__FILE__, __LINE__, hr, 'SetPrimaryBufferFormat', True);
    DXTrace(UnitName, $FFFFFFFF, hr, 'SetPrimaryBufferFormat', True);
    MessageBox(hDlg, 'Error initializing DirectSound.  Sample will now exit.',
                     'DirectSound Sample', MB_OK or MB_ICONERROR );
    EndDialog(hDlg, IDABORT);
    Exit;
  end;

  // Create a timer, so we can check for when the soundbuffer is stopped
  SetTimer(hDlg, 0, 250, nil);

  // Load default wave file
  if (GetWindowsDirectory(strFile, MAX_PATH) = 0) then Exit;
  StrCat(strFile, '\media\ding.wav');

  // Make sure wave file is valid and supported
  if FAILED(ValidateWaveFile(hDlg, strFile)) then
  begin
    // Set the UI controls
    SetDlgItemText(hDlg, IDC_TEXT_FILENAME, '');
    SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'No file loaded.');
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_PLAY), False);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_STOP), False);
    Exit;
  end;

  if SUCCEEDED(g_lpSoundManager.Create(g_lpSound, g_tszFilename,
                                       DSBCAPS_CTRLFX, GUID_NULL)) then
  begin
    g_lpFXManager.Initialize(g_lpSound.GetBuffer(0), True);

    // un-attach the sound buffer
    g_lpFXManager.UnInitialize;

    SAFE_DELETE(g_lpSound);
  end;

  // set UI defaults
  CheckDlgButton(hDlg, IDC_CHECK_LOOP, BST_CHECKED);
  CheckRadioButton(hDlg, IDC_RADIO_CHORUS, IDC_RADIO_REVERB, IDC_RADIO_CHORUS);
  LoadParameterUI(hDlg, DWORD(g_dwCurrentFXType));
  EnablePlayUI(hDlg, True);
end;




//-----------------------------------------------------------------------------
// Name: OnPlaySound()
// Desc: User hit the "Play" button
//-----------------------------------------------------------------------------
function OnPlaySound(hDlg: HWND): HRESULT;
var
  dwCreationFlags: DWORD;
  dwLooped: DWORD;
  i: DWORD;
  bLooped: BOOL;
label
  LFail;
begin
  bLooped := (IsDlgButtonChecked(hDlg, IDC_CHECK_LOOP) = BST_CHECKED);

  // We would only use CTRLFX control on dsound buffer
  dwCreationFlags := DSBCAPS_CTRLFX;

  // Free any previous sound and FXs
  g_lpFXManager.DisableAllFX;
  SAFE_DELETE(g_lpSound);

  // Since the user can change the focus before the sound is played,
  // we need to create the sound buffer every time the play button is pressed

  // Load the wave file into a DirectSound buffer
  Result := g_lpSoundManager.Create(g_lpSound, g_tszFilename,
                                    dwCreationFlags, GUID_NULL);
  if FAILED(Result) then
  begin
    // Not a critical failure, so just update the status
    // DXTrace(__FILE__, __LINE__, Result, 'SetPrimaryBufferFormat', False);
    DXTrace(UnitName, $FFFFFFFF, Result, 'SetPrimaryBufferFormat', False);

    if (Result = DSERR_BUFFERTOOSMALL) then
    begin
      // DSERR_BUFFERTOOSMALL will be returned if the buffer is
      // less than DSBSIZE_FX_MIN (150ms) and the buffer is created
      // with DSBCAPS_CTRLFX.
      SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Wave file is too short (less than 150ms) for effect processing.');
    end else
    begin
      SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Could not create sound buffer.');
    end;

    Result:= S_FALSE;
    Exit;
  end;

  g_lpFXManager.Initialize(g_lpSound.GetBuffer(0) , False);
  for i := IDC_CHECK_CHORUS to IDC_CHECK_REVERB do 
  begin
    if (IsDlgButtonChecked(hDlg, i) = BST_CHECKED)
    then g_lpFXManager.SetFXEnable(i - IDC_CHECK_CHORUS);
  end;


  Result := g_lpFXManager.ActivateFX;
    if FAILED(Result) then goto LFail;
  Result := g_lpFXManager.LoadCurrentFXParameters;
    if FAILED(Result) then goto LFail;

  // Play the sound
  if bLooped then dwLooped := DSBPLAY_LOOPING else dwLooped := 0;
  Result:= g_lpSound.Play(0, dwLooped);
  if FAILED(Result) then
  begin
    // DXTrace(__FILE__, __LINE__, Result, 'Play', False);
    Result:= DXTrace(UnitName, $FFFFFFFF, Result, 'Play', False);
    Exit;
  end;

  // Update the UI controls to show the sound as playing
  EnablePlayUI(hDlg, False);
  SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Sound playing.');

  Result:= S_OK;
  Exit;

LFail:
  // Update the UI controls to show the sound effects could not be loaded
  SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Effects creation failed.  Some effects may not support 8-bit wave files.');

  Result:= S_OK;
end;




//-----------------------------------------------------------------------------
// Name: OnOpenSoundFile()
// Desc: Called when the user requests to open a sound file
//-----------------------------------------------------------------------------
var
  strFileName: array[0..MAX_PATH-1] of Char = '';
  strPath: array[0..MAX_PATH-1] of Char = '';

procedure OnOpenSoundFile(hDlg: HWND);
{$WRITEABLECONST ON}
const
  ofn: TOpenFilename = (
    lStructSize: SizeOf(TOpenFilename);
    hInstance: 0;
    lpstrFilter: 'Wave Files'#0'*.wav'#0'All Files'#0'*.*'#0#0;
    lpstrCustomFilter: nil;
    nMaxCustFilter: 0;
    nFilterIndex: 1;
    nMaxFile: MAX_PATH;
    lpstrFileTitle: nil;
    nMaxFileTitle: 0;
    lpstrTitle: 'Open Sound File';
    Flags: OFN_FILEMUSTEXIST or OFN_HIDEREADONLY;
    nFileExtension: 1;
    lpstrDefExt: '.wav';
  );
{$WRITEABLECONST OFF}
var
  strLastSlash: PChar;
begin
  // Setup the OPENFILENAME structure
  {$IFDEF COMPILER6_UP}
  // Handle Delphi6 case of extended OFN structure
  // (new members are invalid for Win98SE & below OS'es)
  ofn.lStructSize:= 76;

  {$ENDIF}
  ofn.hWndOwner:= hDlg;
  ofn.lpstrFile:= strFileName;
  ofn.lpstrInitialDir:= strPath;

  // Get the default media path (something like C:\WINDOWS\MEDIA)
  if (strPath[0] = #0) then
  begin
    if (GetWindowsDirectory(strPath, MAX_PATH) <> 0) then
    begin
      if StrComp(@strPath[StrLen(strPath)], '\') <> 0 then StrCat(strPath, '\');
      StrCat(strPath, 'MEDIA');
    end;
  end;

  // Update the UI controls to show the sound as loading a file
  EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_PLAY), False);
  EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_STOP), False);
  SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Loading file...');

  // Stop playing the current sound and disable the effects
  g_lpFXManager.DisableAllFX;

  // Display the OpenFileName dialog. Then, try to load the specified file
  if (True <> GetOpenFileName(ofn)) then
  begin
    // Set the UI controls
    SetDlgItemText(hDlg, IDC_TEXT_FILENAME, '');
    SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'No file loaded.');
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_PLAY), False);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_STOP), False);
    Exit;
  end;

  SetDlgItemText(hDlg, IDC_TEXT_FILENAME, '');

  // Make sure wave file is valid and supported
  if FAILED(ValidateWaveFile(hDlg, strFileName)) then
  begin
    // Set the UI controls
    SetDlgItemText(hDlg, IDC_TEXT_FILENAME, '');
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_PLAY), False);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_STOP), False);
    Exit;
  end;

  EnablePlayUI(hDlg, True);

  // Remember the path for next time
  StrCopy(strPath, strFileName);
  strLastSlash := StrRScan(strPath, '\');
  if (strLastSlash <> nil) then strLastSlash[0] := #0;
end;




//-----------------------------------------------------------------------------
// Name: OnTimer()
// Desc: When we think the sound is playing this periodically checks to see if
//       the sound has stopped.  If it has then updates the dialog.
//-----------------------------------------------------------------------------
procedure OnTimer(hDlg: HWND);
begin
  if IsWindowEnabled(GetDlgItem(hDlg, IDC_BUTTON_STOP)) then
  begin
    // We think the sound is playing, so see if it has stopped yet.
    if (g_lpSound <> nil) and (not g_lpSound.IsSoundPlaying) then
    begin
      // Update the UI controls to show the sound as stopped
      EnablePlayUI(hDlg, True);
      SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Sound stopped.');
    end;
  end;
end;




//-----------------------------------------------------------------------------
// Name: EnablePlayUI()
// Desc: Enables or disables the Play UI controls
//-----------------------------------------------------------------------------
procedure EnablePlayUI(hDlg: HWND; bEnable: BOOL);
var
  i: DWORD;
begin
  if (bEnable) then
  begin
    EnableWindow(GetDlgItem(hDlg, IDC_CHECK_LOOP ), True);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_PLAY), True);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_STOP), False);
    SetFocus(    GetDlgItem(hDlg, IDC_BUTTON_PLAY));
  end else
  begin
    EnableWindow(GetDlgItem(hDlg, IDC_CHECK_LOOP ), False);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_PLAY), False);
    EnableWindow(GetDlgItem(hDlg, IDC_BUTTON_STOP), True);
    SetFocus(    GetDlgItem(hDlg, IDC_BUTTON_STOP));
  end;

  for i := IDC_CHECK_CHORUS to IDC_CHECK_REVERB do
    EnableWindow(GetDlgItem(hDlg, i), bEnable);
end;


//-----------------------------------------------------------------------------
// Name: ResetParameterUI( )
// Desc: Blanks the paramter UI
//-----------------------------------------------------------------------------
procedure ResetParameterUI(hwndDlg: HWND);
var
  hwndItem: HWND;
  i: DWORD;
begin
  for i := IDC_PARAM_NAME1 to IDC_PARAM_MAX6 do
  begin
    hwndItem := GetDlgItem(hwndDlg, i);
    SendMessage(hwndItem, WM_SETTEXT, 0, Integer(PChar('- - -')));
    EnableWindow(hwndItem, False);
  end;

  i := IDC_SLIDER1;
  // for ( i = IDC_SLIDER1; i <= IDC_SLIDER6; i += 5 )
  while (i <= IDC_SLIDER6) do 
  begin
    PostMessage(GetDlgItem(hwndDlg, i), TBM_SETRANGEMIN, 0{FALSE}, DEFAULT_SLIDER_MIN);
    PostMessage(GetDlgItem(hwndDlg, i), TBM_SETRANGEMAX, 0{FALSE}, DEFAULT_SLIDER_MAX);
    PostMessage(GetDlgItem(hwndDlg, i), TBM_SETLINESIZE, 0{FALSE}, DEFAULT_SLIDER_INC);
    PostMessage(GetDlgItem(hwndDlg, i), TBM_SETPOS, 0{TRUE}, 0);
    Inc(i, 5);
  end;

  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_TRIANGLE), False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_SQUARE),   False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_SINE),     False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_FRAME_WAVEFORM), False);

  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_NEG_180),  False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_NEG_90),   False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_ZERO),     False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_90),       False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_180),      False);
  EnableWindow(GetDlgItem(hwndDlg, IDC_FRAME_PHASE),    False);
end;


//-----------------------------------------------------------------------------
// Name: EnableSingleParameter
// Desc:
//-----------------------------------------------------------------------------
procedure EnableSingleParameter(hwndDlg: HWND; resID: DWORD);
var
  i: DWORD;
begin
  for i := resID to resID + 5 - 1 do
    EnableWindow(GetDlgItem(hwndDlg, i), True);
end;


//-----------------------------------------------------------------------------
// Name: PosSlider
// Desc:
//-----------------------------------------------------------------------------
procedure PosSlider(hwndDlg: HWND; dwSlider: DWORD; val: Single; min: Single; max: Single;
                    fSliderInc: Single);
var
  hwndSlider: HWND;
  lSliderInc: Longint;
  res: Single;
  pos: Longint;
begin
  hwndSlider := GetDlgItem(hwndDlg, dwSlider);

  lSliderInc := Trunc((fSliderInc / (max - min)) * DEFAULT_SLIDER_MAX);
  PostMessage(hwndSlider, TBM_SETLINESIZE, 0{FALSE}, lSliderInc);

  res := (val - min) / (max - min);
  if (res < 0.0) then res := 0.0
  else
    if (res > 1.0) then res := 1.0;

  pos := Trunc(res * DEFAULT_SLIDER_MAX);
  PostMessage(hwndSlider, TBM_SETPOS, 1{TRUE}, pos);
end;


//-----------------------------------------------------------------------------
// Name: LoadSingleParameter
// Desc:
//-----------------------------------------------------------------------------
procedure LoadSingleParameter(hwndDlg: HWND; id: DWORD; const tszName: PChar; val: Single;
    min: Single; max: Single; fSliderInc: Single = 0.1;
    dwPrecision: DWORD = 1);
var
  tszstr: array[0..MAX_PATH-1] of Char;
begin
  // wet dry mix
  EnableSingleParameter(hwndDlg, id);
  SendMessage(GetDlgItem(hwndDlg, id), WM_SETTEXT, 0, LPARAM(tszName));

  case dwPrecision of
    0:
    begin
      StrFmt(tszstr, '%.0f', [val]);
      SendMessage(GetDlgItem(hwndDlg, id + 1), WM_SETTEXT, 0, LPARAM(@tszstr));
      StrFmt(tszstr, '%.0f', [min]);
      SendMessage(GetDlgItem(hwndDlg, id + 3), WM_SETTEXT, 0, LPARAM(@tszstr));
      StrFmt(tszstr, '%.0f', [max]);
      SendMessage(GetDlgItem(hwndDlg, id + 4), WM_SETTEXT, 0, LPARAM(@tszstr));
    end;
    1:
    begin
      StrFmt(tszstr, '%.1f', [val]);
      SendMessage(GetDlgItem(hwndDlg, id + 1), WM_SETTEXT, 0, LPARAM(@tszstr));
      StrFmt(tszstr, '%.1f', [min]);
      SendMessage(GetDlgItem(hwndDlg, id + 3), WM_SETTEXT, 0, LPARAM(@tszstr));
      StrFmt(tszstr, '%.1f', [max]);
      SendMessage(GetDlgItem(hwndDlg, id + 4), WM_SETTEXT, 0, LPARAM(@tszstr));
    end;
    else
    // 2:
      StrFmt(tszstr, '%.2f', [val]);
      SendMessage(GetDlgItem(hwndDlg, id + 1), WM_SETTEXT, 0, LPARAM(@tszstr));
      StrFmt(tszstr, '%.2f', [min]);
      SendMessage(GetDlgItem(hwndDlg, id + 3), WM_SETTEXT, 0, LPARAM(@tszstr));
      StrFmt(tszstr, '%.2f', [max]);
      SendMessage(GetDlgItem(hwndDlg, id + 4), WM_SETTEXT, 0, LPARAM(@tszstr));
  end;

  PosSlider(hwndDlg, id + 2, val, min, max, fSliderInc);
end;


//-----------------------------------------------------------------------------
// Name: SaveSingleParameter
// Desc:
//-----------------------------------------------------------------------------
procedure SaveSingleParameter(hwndDlg: HWND; id: DWORD; out val: Single; min: Single; max: Single; dwPrecision: DWORD = 1);
var
  tszstr: array[0..MAX_PATH-1] of Char;
  pos: DWORD;
  percent: Single;
begin
  pos := SendMessage(GetDlgItem(hwndDlg, id + 2), TBM_GETPOS, 0, 0);

  percent := (pos - DEFAULT_SLIDER_MIN) / (DEFAULT_SLIDER_MAX - DEFAULT_SLIDER_MIN);
  val := percent * ( max - min ) + min;

  case dwPrecision of
    0: StrFmt(tszstr, '%.0f', [val]);
    1: StrFmt(tszstr, '%.1f', [val]);
   else
    {2:} StrFmt(tszstr, '%.2f', [val]);
  end;

  SendMessage(GetDlgItem(hwndDlg, id + 1), WM_SETTEXT, 0, LPARAM(@tszstr));
end;


//-----------------------------------------------------------------------------
// Name: LoadWaveformRadio
// Desc:
//-----------------------------------------------------------------------------
procedure LoadWaveformRadio(hwndDlg: HWND; waveform: Longint; triangle: Longint; square: Longint; sine: Longint);
begin
  if (waveform = triangle) then
    CheckRadioButton(hwndDlg, IDC_RADIO_TRIANGLE, IDC_RADIO_SINE, IDC_RADIO_TRIANGLE)
  else if (waveform = square) then
    CheckRadioButton(hwndDlg, IDC_RADIO_TRIANGLE, IDC_RADIO_SINE, IDC_RADIO_SQUARE)
  else if (waveform = sine) then
    CheckRadioButton( hwndDlg, IDC_RADIO_TRIANGLE, IDC_RADIO_SINE, IDC_RADIO_SINE);

  if (triangle >= 0) then
    EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_TRIANGLE), True);
  if (square >= 0) then
    EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_SQUARE), True);
  if (sine >= 0) then
    EnableWindow(GetDlgItem(hwndDlg, IDC_RADIO_SINE), True);

  EnableWindow(GetDlgItem(hwndDlg, IDC_FRAME_WAVEFORM), (triangle + square + sine <> -3));
end;


//-----------------------------------------------------------------------------
// Name: LoadPhaseRadio
// Desc:
//-----------------------------------------------------------------------------
procedure LoadPhaseRadio(hwndDlg: HWND; phase: Longint; neg180: Longint; neg90: Longint; zero: Longint; pos90: Longint; pos180: Longint);
var
  i: Integer;
begin
  for i := IDC_RADIO_NEG_180 to IDC_RADIO_180 do
    EnableWindow(GetDlgItem(hwndDlg, i), True);

  EnableWindow(GetDlgItem(hwndDlg, IDC_FRAME_PHASE), True);

  if (phase = neg180) then
    CheckRadioButton(hwndDlg, IDC_RADIO_NEG_180, IDC_RADIO_90, IDC_RADIO_NEG_180)
  else if (phase = neg90) then
    CheckRadioButton(hwndDlg, IDC_RADIO_NEG_180, IDC_RADIO_90, IDC_RADIO_NEG_90)
  else if (phase = zero) then
    CheckRadioButton(hwndDlg, IDC_RADIO_NEG_180, IDC_RADIO_90, IDC_RADIO_ZERO)
  else if (phase = pos90) then
    CheckRadioButton(hwndDlg, IDC_RADIO_NEG_180, IDC_RADIO_90, IDC_RADIO_90)
  else if (phase = pos180) then
    CheckRadioButton(hwndDlg, IDC_RADIO_NEG_180, IDC_RADIO_90, IDC_RADIO_180)
end;


//-----------------------------------------------------------------------------
// Name: LoadParameterUI( )
// Desc: loads the paramter ui for particular effect
//-----------------------------------------------------------------------------
procedure LoadParameterUI(hwndDlg: HWND; dwFXType: DWORD);
var
  tszstr: array[0..MAX_PATH-1] of Char;
begin
  // reset the parameter ui
  ResetParameterUI(hwndDlg);

  StrFmt(tszstr, 'Parameters for [ %s ]', [g_tszFXNames[dwFXType]]);
  SetDlgItemText(hwndDlg, IDC_FRAME, @tszstr);

  case TESFXType(dwFXType) of
    eSFX_chorus:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Wet/Dry Mix (%)', g_lpFXManager.m_paramsChorus.fWetDryMix, DSFXCHORUS_WETDRYMIX_MIN, DSFXCHORUS_WETDRYMIX_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Depth (%)', g_lpFXManager.m_paramsChorus.fDepth, DSFXCHORUS_DEPTH_MIN, DSFXCHORUS_DEPTH_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'Feedback (%)', g_lpFXManager.m_paramsChorus.fFeedback, DSFXCHORUS_FEEDBACK_MIN, DSFXCHORUS_FEEDBACK_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME4, 'Frequency (Hz)', g_lpFXManager.m_paramsChorus.fFrequency, DSFXCHORUS_FREQUENCY_MIN, DSFXCHORUS_FREQUENCY_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME5, 'Delay (ms)', g_lpFXManager.m_paramsChorus.fDelay, DSFXCHORUS_DELAY_MIN, DSFXCHORUS_DELAY_MAX);
      LoadWaveformRadio(hwndDlg, g_lpFXManager.m_paramsChorus.lWaveform, DSFXCHORUS_WAVE_TRIANGLE, -1, DSFXCHORUS_WAVE_SIN);
      LoadPhaseRadio(hwndDlg, g_lpFXManager.m_paramsChorus.lPhase, DSFXCHORUS_PHASE_NEG_180, DSFXCHORUS_PHASE_NEG_90, DSFXCHORUS_PHASE_ZERO, DSFXCHORUS_PHASE_90, DSFXCHORUS_PHASE_180);
    end;

    eSFX_compressor:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Gain (dB)', g_lpFXManager.m_paramsCompressor.fGain, DSFXCOMPRESSOR_GAIN_MIN, DSFXCOMPRESSOR_GAIN_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Attack (ms)', g_lpFXManager.m_paramsCompressor.fAttack, DSFXCOMPRESSOR_ATTACK_MIN, DSFXCOMPRESSOR_ATTACK_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'Release (ms)', g_lpFXManager.m_paramsCompressor.fRelease, DSFXCOMPRESSOR_RELEASE_MIN, DSFXCOMPRESSOR_RELEASE_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME4, 'Threshold (dB)', g_lpFXManager.m_paramsCompressor.fThreshold, DSFXCOMPRESSOR_THRESHOLD_MIN, DSFXCOMPRESSOR_THRESHOLD_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME5, 'Ratio (x:1)', g_lpFXManager.m_paramsCompressor.fRatio, DSFXCOMPRESSOR_RATIO_MIN, DSFXCOMPRESSOR_RATIO_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME6, 'Predelay (ms)', g_lpFXManager.m_paramsCompressor.fPredelay, DSFXCOMPRESSOR_PREDELAY_MIN, DSFXCOMPRESSOR_PREDELAY_MAX, 0.05, 2);
    end;

    eSFX_distortion:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Gain (dB)', g_lpFXManager.m_paramsDistortion.fGain, DSFXDISTORTION_GAIN_MIN, DSFXDISTORTION_GAIN_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Edge (%)', g_lpFXManager.m_paramsDistortion.fEdge, DSFXDISTORTION_EDGE_MIN, DSFXDISTORTION_EDGE_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'PostEQ Center Freq (Hz)', g_lpFXManager.m_paramsDistortion.fPostEQCenterFrequency, DSFXDISTORTION_POSTEQCENTERFREQUENCY_MIN, DSFXDISTORTION_POSTEQCENTERFREQUENCY_MAX, 1.0, 0);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME4, 'PostEQ Bandwidth (Hz)', g_lpFXManager.m_paramsDistortion.fPostEQBandwidth, DSFXDISTORTION_POSTEQBANDWIDTH_MIN, DSFXDISTORTION_POSTEQBANDWIDTH_MAX, 1.0, 0);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME5, 'PreLowpass Cutoff (Hz)', g_lpFXManager.m_paramsDistortion.fPreLowpassCutoff, DSFXDISTORTION_PRELOWPASSCUTOFF_MIN, DSFXDISTORTION_PRELOWPASSCUTOFF_MAX, 1.0, 0);
    end;

    eSFX_echo:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Wet/Dry Mix (%)', g_lpFXManager.m_paramsEcho.fWetDryMix, DSFXECHO_WETDRYMIX_MIN, DSFXECHO_WETDRYMIX_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Feedback (%)', g_lpFXManager.m_paramsEcho.fFeedback, DSFXECHO_FEEDBACK_MIN, DSFXECHO_FEEDBACK_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'Left Delay (ms)', g_lpFXManager.m_paramsEcho.fLeftDelay, DSFXECHO_LEFTDELAY_MIN, DSFXECHO_LEFTDELAY_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME4, 'Right Delay (ms)', g_lpFXManager.m_paramsEcho.fRightDelay, DSFXECHO_RIGHTDELAY_MIN, DSFXECHO_RIGHTDELAY_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME5, 'Pan Delay (bool)', g_lpFXManager.m_paramsEcho.lPanDelay, DSFXECHO_PANDELAY_MIN, DSFXECHO_PANDELAY_MAX);
    end;

    eSFX_flanger:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Wet/Dry Mix (%)', g_lpFXManager.m_paramsFlanger.fWetDryMix, DSFXFLANGER_WETDRYMIX_MIN, DSFXFLANGER_WETDRYMIX_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Depth (%)', g_lpFXManager.m_paramsFlanger.fDepth, DSFXFLANGER_DEPTH_MIN, DSFXFLANGER_DEPTH_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'Feedback (%)', g_lpFXManager.m_paramsFlanger.fFeedback, DSFXFLANGER_FEEDBACK_MIN, DSFXFLANGER_FEEDBACK_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME4, 'Frequency (Hz)', g_lpFXManager.m_paramsFlanger.fFrequency, DSFXFLANGER_FREQUENCY_MIN, DSFXFLANGER_FREQUENCY_MAX, 0.01, 2);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME5, 'Delay (ms)', g_lpFXManager.m_paramsFlanger.fDelay, DSFXFLANGER_DELAY_MIN, DSFXFLANGER_DELAY_MAX, 0.01, 2);
      LoadWaveformRadio(hwndDlg, g_lpFXManager.m_paramsFlanger.lWaveform, DSFXFLANGER_WAVE_TRIANGLE, -1, DSFXFLANGER_WAVE_SIN);
      LoadPhaseRadio(hwndDlg, g_lpFXManager.m_paramsFlanger.lPhase, DSFXFLANGER_PHASE_NEG_180, DSFXFLANGER_PHASE_NEG_90, DSFXFLANGER_PHASE_ZERO, DSFXFLANGER_PHASE_90, DSFXFLANGER_PHASE_180);
    end;

    eSFX_gargle:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Rate (Hz)', g_lpFXManager.m_paramsGargle.dwRateHz, DSFXGARGLE_RATEHZ_MIN, DSFXGARGLE_RATEHZ_MAX);
      LoadWaveformRadio(hwndDlg, g_lpFXManager.m_paramsGargle.dwWaveShape, DSFXGARGLE_WAVE_TRIANGLE, DSFXGARGLE_WAVE_SQUARE, -1);
    end;

    eSFX_parameq:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'Center Freq (Hz)', g_lpFXManager.m_paramsParamEq.fCenter, DSFXPARAMEQ_CENTER_MIN, DSFXPARAMEQ_CENTER_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Bandwidth (Hz)', g_lpFXManager.m_paramsParamEq.fBandwidth, DSFXPARAMEQ_BANDWIDTH_MIN, DSFXPARAMEQ_BANDWIDTH_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'Gain (dB)', g_lpFXManager.m_paramsParamEq.fGain, DSFXPARAMEQ_GAIN_MIN, DSFXPARAMEQ_GAIN_MAX);
    end;

    eSFX_reverb:
    begin
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME1, 'In Gain (dB)', g_lpFXManager.m_paramsReverb.fInGain, DSFX_WAVESREVERB_INGAIN_MIN, DSFX_WAVESREVERB_INGAIN_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME2, 'Reverb Mix (dB)', g_lpFXManager.m_paramsReverb.fReverbMix, DSFX_WAVESREVERB_REVERBMIX_MIN, DSFX_WAVESREVERB_REVERBMIX_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME3, 'Reverb Time (ms)', g_lpFXManager.m_paramsReverb.fReverbTime, DSFX_WAVESREVERB_REVERBTIME_MIN, DSFX_WAVESREVERB_REVERBTIME_MAX);
      LoadSingleParameter(hwndDlg, IDC_PARAM_NAME4, 'HighFreq RT Ratio (x:1)', g_lpFXManager.m_paramsReverb.fHighFreqRTRatio, DSFX_WAVESREVERB_HIGHFREQRTRATIO_MIN, DSFX_WAVESREVERB_HIGHFREQRTRATIO_MAX, 0.01, 2);
    end;
  end;
end;



//-----------------------------------------------------------------------------
// Name: OnEffectChanged()
// Desc: Called when the UI prompted an effect change
//-----------------------------------------------------------------------------
procedure OnEffectChanged(hwndDlg: HWND);
var
  fTemp: Single;
  fRateHz: Single;
begin
  case g_dwCurrentFXType of
    eSFX_chorus:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsChorus.fWetDryMix, DSFXCHORUS_WETDRYMIX_MIN, DSFXCHORUS_WETDRYMIX_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsChorus.fDepth, DSFXCHORUS_DEPTH_MIN, DSFXCHORUS_DEPTH_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsChorus.fFeedback, DSFXCHORUS_FEEDBACK_MIN, DSFXCHORUS_FEEDBACK_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME4, g_lpFXManager.m_paramsChorus.fFrequency, DSFXCHORUS_FREQUENCY_MIN, DSFXCHORUS_FREQUENCY_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME5, g_lpFXManager.m_paramsChorus.fDelay, DSFXCHORUS_DELAY_MIN, DSFXCHORUS_DELAY_MAX);

      if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_TRIANGLE) = BST_CHECKED)
      then g_lpFXManager.m_paramsChorus.lWaveform := DSFXCHORUS_WAVE_TRIANGLE
      else g_lpFXManager.m_paramsChorus.lWaveform := DSFXCHORUS_WAVE_SIN;

      if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_NEG_180) = BST_CHECKED) then
        g_lpFXManager.m_paramsChorus.lPhase := DSFXCHORUS_PHASE_NEG_180
      else if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_NEG_90) = BST_CHECKED) then
        g_lpFXManager.m_paramsChorus.lPhase := DSFXCHORUS_PHASE_NEG_90
      else if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_ZERO) = BST_CHECKED) then
        g_lpFXManager.m_paramsChorus.lPhase := DSFXCHORUS_PHASE_ZERO
      else if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_90) = BST_CHECKED) then
        g_lpFXManager.m_paramsChorus.lPhase := DSFXCHORUS_PHASE_90
      else
        g_lpFXManager.m_paramsChorus.lPhase := DSFXCHORUS_PHASE_180;

      if (g_lpFXManager.m_lpChorus <> nil) then
          g_lpFXManager.m_lpChorus.SetAllParameters(g_lpFXManager.m_paramsChorus);
    end;

    eSFX_compressor:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsCompressor.fGain, DSFXCOMPRESSOR_GAIN_MIN, DSFXCOMPRESSOR_GAIN_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsCompressor.fAttack, DSFXCOMPRESSOR_ATTACK_MIN, DSFXCOMPRESSOR_ATTACK_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsCompressor.fRelease, DSFXCOMPRESSOR_RELEASE_MIN, DSFXCOMPRESSOR_RELEASE_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME4, g_lpFXManager.m_paramsCompressor.fThreshold, DSFXCOMPRESSOR_THRESHOLD_MIN, DSFXCOMPRESSOR_THRESHOLD_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME5, g_lpFXManager.m_paramsCompressor.fRatio, DSFXCOMPRESSOR_RATIO_MIN, DSFXCOMPRESSOR_RATIO_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME6, g_lpFXManager.m_paramsCompressor.fPredelay, DSFXCOMPRESSOR_PREDELAY_MIN, DSFXCOMPRESSOR_PREDELAY_MAX, 2);

      if (g_lpFXManager.m_lpCompressor <> nil) then
        g_lpFXManager.m_lpCompressor.SetAllParameters(g_lpFXManager.m_paramsCompressor);
    end;

    eSFX_distortion:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsDistortion.fGain, DSFXDISTORTION_GAIN_MIN, DSFXDISTORTION_GAIN_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsDistortion.fEdge, DSFXDISTORTION_EDGE_MIN, DSFXDISTORTION_EDGE_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsDistortion.fPostEQCenterFrequency, DSFXDISTORTION_POSTEQCENTERFREQUENCY_MIN, DSFXDISTORTION_POSTEQCENTERFREQUENCY_MAX, 0);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME4, g_lpFXManager.m_paramsDistortion.fPostEQBandwidth, DSFXDISTORTION_POSTEQBANDWIDTH_MIN, DSFXDISTORTION_POSTEQBANDWIDTH_MAX, 0);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME5, g_lpFXManager.m_paramsDistortion.fPreLowpassCutoff, DSFXDISTORTION_PRELOWPASSCUTOFF_MIN, DSFXDISTORTION_PRELOWPASSCUTOFF_MAX, 0);

      if (g_lpFXManager.m_lpDistortion <> nil) then
        g_lpFXManager.m_lpDistortion.SetAllParameters(g_lpFXManager.m_paramsDistortion);
    end;

    eSFX_echo:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsEcho.fWetDryMix, DSFXECHO_WETDRYMIX_MIN, DSFXECHO_WETDRYMIX_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsEcho.fFeedback, DSFXECHO_FEEDBACK_MIN, DSFXECHO_FEEDBACK_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsEcho.fLeftDelay, DSFXECHO_LEFTDELAY_MIN, DSFXECHO_LEFTDELAY_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME4, g_lpFXManager.m_paramsEcho.fRightDelay, DSFXECHO_RIGHTDELAY_MIN, DSFXECHO_RIGHTDELAY_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME5, fTemp, DSFXECHO_PANDELAY_MIN, DSFXECHO_PANDELAY_MAX);
      g_lpFXManager.m_paramsEcho.lPanDelay := Trunc(fTemp);

      if (g_lpFXManager.m_lpEcho <> nil) then
        g_lpFXManager.m_lpEcho.SetAllParameters(g_lpFXManager.m_paramsEcho);
    end;

    eSFX_flanger:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsFlanger.fWetDryMix, DSFXFLANGER_WETDRYMIX_MIN, DSFXFLANGER_WETDRYMIX_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsFlanger.fDepth, DSFXFLANGER_DEPTH_MIN, DSFXFLANGER_DEPTH_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsFlanger.fFeedback, DSFXFLANGER_FEEDBACK_MIN, DSFXFLANGER_FEEDBACK_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME4, g_lpFXManager.m_paramsFlanger.fFrequency, DSFXFLANGER_FREQUENCY_MIN, DSFXFLANGER_FREQUENCY_MAX, 2);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME5, g_lpFXManager.m_paramsFlanger.fDelay, DSFXFLANGER_DELAY_MIN, DSFXFLANGER_DELAY_MAX, 2);

      if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_TRIANGLE) = BST_CHECKED) then 
          g_lpFXManager.m_paramsFlanger.lWaveform := DSFXFLANGER_WAVE_TRIANGLE
      else
          g_lpFXManager.m_paramsFlanger.lWaveform := DSFXFLANGER_WAVE_SIN;

      if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_NEG_180) = BST_CHECKED) then
        g_lpFXManager.m_paramsFlanger.lPhase := DSFXFLANGER_PHASE_NEG_180
      else if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_NEG_90) = BST_CHECKED) then
        g_lpFXManager.m_paramsFlanger.lPhase := DSFXFLANGER_PHASE_NEG_90
      else if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_ZERO) = BST_CHECKED) then
        g_lpFXManager.m_paramsFlanger.lPhase := DSFXFLANGER_PHASE_ZERO
      else if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_90) = BST_CHECKED) then
        g_lpFXManager.m_paramsFlanger.lPhase := DSFXFLANGER_PHASE_90
      else
        g_lpFXManager.m_paramsFlanger.lPhase := DSFXFLANGER_PHASE_180;

      if (g_lpFXManager.m_lpFlanger <> nil) then
        g_lpFXManager.m_lpFlanger.SetAllParameters(g_lpFXManager.m_paramsFlanger);
    end;

    eSFX_gargle:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, fRateHz, DSFXGARGLE_RATEHZ_MIN, DSFXGARGLE_RATEHZ_MAX);
      g_lpFXManager.m_paramsGargle.dwRateHz := Trunc(fRateHz);

      if (IsDlgButtonChecked(hwndDlg, IDC_RADIO_TRIANGLE) = BST_CHECKED) then
        g_lpFXManager.m_paramsGargle.dwWaveShape := DSFXGARGLE_WAVE_TRIANGLE
      else
        g_lpFXManager.m_paramsGargle.dwWaveShape := DSFXGARGLE_WAVE_SQUARE;

      if (g_lpFXManager.m_lpGargle <> nil) then
        g_lpFXManager.m_lpGargle.SetAllParameters(g_lpFXManager.m_paramsGargle);
    end;

    eSFX_parameq:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsParamEq.fCenter, DSFXPARAMEQ_CENTER_MIN, DSFXPARAMEQ_CENTER_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsParamEq.fBandwidth, DSFXPARAMEQ_BANDWIDTH_MIN, DSFXPARAMEQ_BANDWIDTH_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsParamEq.fGain, DSFXPARAMEQ_GAIN_MIN, DSFXPARAMEQ_GAIN_MAX);

      if (g_lpFXManager.m_lpParamEq <> nil) then
        g_lpFXManager.m_lpParamEq.SetAllParameters(g_lpFXManager.m_paramsParamEq);
    end;

    eSFX_reverb:
    begin
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME1, g_lpFXManager.m_paramsReverb.fInGain, DSFX_WAVESREVERB_INGAIN_MIN, DSFX_WAVESREVERB_INGAIN_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME2, g_lpFXManager.m_paramsReverb.fReverbMix, DSFX_WAVESREVERB_REVERBMIX_MIN, DSFX_WAVESREVERB_REVERBMIX_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME3, g_lpFXManager.m_paramsReverb.fReverbTime, DSFX_WAVESREVERB_REVERBTIME_MIN, DSFX_WAVESREVERB_REVERBTIME_MAX);
      SaveSingleParameter(hwndDlg, IDC_PARAM_NAME4, g_lpFXManager.m_paramsReverb.fHighFreqRTRatio, DSFX_WAVESREVERB_HIGHFREQRTRATIO_MIN, DSFX_WAVESREVERB_HIGHFREQRTRATIO_MAX, 2);

      if (g_lpFXManager.m_lpReverb <> nil) then
        g_lpFXManager.m_lpReverb.SetAllParameters(g_lpFXManager.m_paramsReverb);
    end;
  end;
end;


//-----------------------------------------------------------------------------
// Name: ValidateWaveFile()
// Desc: Open the wave file with the helper
//       class CWaveFile to make sure it is valid
//-----------------------------------------------------------------------------
function ValidateWaveFile(hDlg: HWND; strFileName: PChar): HRESULT;
var
  waveFile: CWaveFile;
  hFile: THandle;
  dwFileSizeHigh: DWORD;
  dwFileSize: DWORD;
  pwfx: PWaveFormatEx;
begin
  waveFile:= CWaveFile.Create;
{$IFDEF SUPPORTS_EXCEPTIONS}
  try
{$ELSE}
  begin
{$ENDIF}
    if (DWORD(-1) = GetFileAttributes(strFileName)) then
    begin
      Result:= E_FAIL;
      Exit;
    end;

    // Verify the file is small
    hFile := CreateFile(strFileName, 0, FILE_SHARE_READ, nil, OPEN_EXISTING, 0, 0);
    if (hFile <> 0) then
    begin
      // If you try to open a 100MB wav file, you could run out of system memory with this
      // sample cause it puts all of it into a large buffer.  If you need to do this, then
      // see the "StreamData" sample to stream the data from the file into a sound buffer.
      dwFileSizeHigh := 0;
      dwFileSize := GetFileSize(hFile, @dwFileSizeHigh);
      CloseHandle(hFile);

      if (dwFileSizeHigh <> 0) or (dwFileSize > 1000000) then
      begin
        SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'File too large.  You should stream large files.');
        Result:= E_FAIL;
        Exit;
      end;
    end;

    // Load the wave file
    Result := waveFile.Open(strFileName, nil, WAVEFILE_READ);
    if FAILED(Result) then
    begin
      waveFile.Close;
      SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Bad wave file.');
      // DXTrace(__FILE__, __LINE__, Result, 'Open', True);
      Result:= DXTrace(UnitName, $FFFFFFFF, Result, 'Open', True);
      Exit;
    end
    else // The load call succeeded
    begin
      pwfx := waveFile.GetFormat;
      if (pwfx.wFormatTag <> WAVE_FORMAT_PCM) and (pwfx.wFormatTag <> WAVE_FORMAT_IEEE_FLOAT) then
      begin
        // Sound must be PCM or IEEE FLOAT when using DSBCAPS_CTRLFX
        SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'Wave file must be PCM or IEEE FLOAT for effects control.');
        SetDlgItemText(hDlg, IDC_TEXT_FILENAME, '');
        Result:= E_FAIL;
        Exit;
      end;

      // Update the UI controls to show the sound as the file is loaded
      waveFile.Close;

      //EnablePlayUI( hDlg, TRUE );
      SetDlgItemText(hDlg, IDC_TEXT_FILENAME, strFileName);
      SetDlgItemText(hDlg, IDC_TEXT_STATUS, 'File loaded.');
      StrCopy(g_tszFilename, strFileName);
      Result:= S_OK;
    end;
{$IFDEF SUPPORTS_EXCEPTIONS}
  finally
{$ENDIF}
    waveFile.Free;
  end;
end;



{ CSoundFXManager }

//-----------------------------------------------------------------------------
// Name: CSoundFXManager( )
// Desc: constructor
//-----------------------------------------------------------------------------
constructor CSoundFXManager.Create;
begin
  m_lpChorus := nil;
  m_lpCompressor := nil;
  m_lpDistortion := nil;
  m_lpEcho := nil;
  m_lpFlanger := nil;
  m_lpGargle := nil;
  m_lpParamEq := nil;
  m_lpReverb := nil;

  ZeroMemory(@m_paramsChorus, SizeOf(TDSFXChorus));
  ZeroMemory(@m_paramsCompressor, SizeOf(TDSFXCompressor));
  ZeroMemory(@m_paramsDistortion, SizeOf(TDSFXDistortion));
  ZeroMemory(@m_paramsFlanger, SizeOf(TDSFXFlanger));
  ZeroMemory(@m_paramsEcho, SizeOf(TDSFXEcho));
  ZeroMemory(@m_paramsGargle, SizeOf(TDSFXGargle));
  ZeroMemory(@m_paramsParamEq, SizeOf(TDSFXParamEq));
  ZeroMemory(@m_paramsReverb, SizeOf(TDSFXWavesReverb));

  m_dwNumFX := 0;
  ZeroMemory(@m_rgFxDesc, SizeOf(TDSEffectDesc) * eNUM_SFX);
  ZeroMemory(@m_rgRefGuids, SizeOf(PGUID) * eNUM_SFX);
  ZeroMemory(@m_rgPtrs, SizeOf(PPointer) * eNUM_SFX);
  ZeroMemory(@m_rgLoaded, SizeOf(BOOL) * eNUM_SFX);

  m_lpDSB8 := nil;
end;

//-----------------------------------------------------------------------------
// Name: ~CSoundFXManager( )
// Desc: destructor
//-----------------------------------------------------------------------------
destructor CSoundFXManager.Destroy;
begin
  // free any effects
  DisableAllFX();

  // release the DirectSoundBuffer
  SAFE_RELEASE( m_lpDSB8);

  inherited;
end;


//-----------------------------------------------------------------------------
// Name: Initialize( )
// Desc: associates a DirectSoundBuffer with the manager, any effects
//       enabled in the old DirectSoundBuffer will be disabled,
//       and the effect objects released
//-----------------------------------------------------------------------------
function CSoundFXManager.Initialize(lpDSB: IDirectSoundBuffer;
  bLoadDefaultParamValues: BOOL): HRESULT;
begin
  if (m_lpDSB8 <> nil) then
  begin
    // release the effect for the previously associated sound buffers
    DisableAllFX;
    SAFE_RELEASE(m_lpDSB8);
  end;

  if (nil = lpDSB) then
  begin
    Result:= S_OK;
    Exit;
  end;

  // get the interface
  Result := lpDSB.QueryInterface(IID_IDirectSoundBuffer8, m_lpDSB8);
  if FAILED(Result) then Exit;

  if (bLoadDefaultParamValues) then LoadDefaultParamValues;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: UnInitialize( )
// Desc: the manager goes back to default state, the effects params, however
//       will not be reset
//-----------------------------------------------------------------------------
function CSoundFXManager.UnInitialize: HRESULT;
begin
  Initialize(nil, False);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: LoadDefaultParamValues( )
// Desc: loads the default param value for each effect
//-----------------------------------------------------------------------------
function CSoundFXManager.LoadDefaultParamValues: HRESULT;
var
  i: DWORD;
begin
  if (nil = m_lpDSB8) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  for i := DWORD(eSFX_chorus) to eNUM_SFX - 1 do SetFXEnable(i);

  ActivateFX();

  if (m_lpChorus <> nil) then
    m_lpChorus.GetAllParameters(m_paramsChorus);

  if (m_lpCompressor <> nil) then
    m_lpCompressor.GetAllParameters(m_paramsCompressor);

  if (m_lpDistortion <> nil) then
    m_lpDistortion.GetAllParameters(m_paramsDistortion);

  if (m_lpEcho <> nil) then
    m_lpEcho.GetAllParameters(m_paramsEcho);

  if (m_lpFlanger <> nil) then
    m_lpFlanger.GetAllParameters(m_paramsFlanger);

  if (m_lpGargle <> nil) then
    m_lpGargle.GetAllParameters(m_paramsGargle);

  if (m_lpParamEq <> nil) then
    m_lpParamEq.GetAllParameters(m_paramsParamEq);

  if (m_lpReverb <> nil) then
    m_lpReverb.GetAllParameters(m_paramsReverb);

  DisableAllFX;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: LoadCurrentFXParameters
// Desc: loads the default param value for each effect
//-----------------------------------------------------------------------------
function CSoundFXManager.LoadCurrentFXParameters: HRESULT;
begin
  if (m_lpChorus <> nil) then
    m_lpChorus.SetAllParameters(m_paramsChorus);

  if (m_lpCompressor <> nil) then
    m_lpCompressor.SetAllParameters(m_paramsCompressor);

  if (m_lpDistortion <> nil) then
    m_lpDistortion.SetAllParameters(m_paramsDistortion);

  if (m_lpEcho <> nil) then
    m_lpEcho.SetAllParameters(m_paramsEcho);

  if (m_lpFlanger <> nil) then
    m_lpFlanger.SetAllParameters(m_paramsFlanger);

  if (m_lpGargle <> nil) then
    m_lpGargle.SetAllParameters(m_paramsGargle);

  if (m_lpParamEq <> nil) then
    m_lpParamEq.SetAllParameters(m_paramsParamEq);

  if (m_lpReverb <> nil) then
    m_lpReverb.SetAllParameters(m_paramsReverb);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: SetFXEnable( )
// Desc: enables a sound effect for the sound buffer associated with this
//-----------------------------------------------------------------------------
function CSoundFXManager.SetFXEnable(esfxType: DWORD): HRESULT;
begin
  if (esfxType >= eNUM_SFX) then
  begin
    Result := E_FAIL;
    Exit;
  end;

  if (m_rgLoaded[esfxType]) then
  begin
    Result := S_FALSE;
    Exit;
  end else
    m_rgLoaded[esfxType] := True;

  case TESFXType(esfxType) of
    eSFX_chorus:     Result := EnableGenericFX(GUID_DSFX_STANDARD_CHORUS,     IID_IDirectSoundFXChorus8,      @m_lpChorus);
    eSFX_compressor: Result := EnableGenericFX(GUID_DSFX_STANDARD_COMPRESSOR, IID_IDirectSoundFXCompressor8,  @m_lpCompressor);
    eSFX_distortion: Result := EnableGenericFX(GUID_DSFX_STANDARD_DISTORTION, IID_IDirectSoundFXDistortion8,  @m_lpDistortion);
    eSFX_echo:       Result := EnableGenericFX(GUID_DSFX_STANDARD_ECHO,       IID_IDirectSoundFXEcho8,        @m_lpEcho);
    eSFX_flanger:    Result := EnableGenericFX(GUID_DSFX_STANDARD_FLANGER,    IID_IDirectSoundFXFlanger8,     @m_lpFlanger);
    eSFX_gargle:     Result := EnableGenericFX(GUID_DSFX_STANDARD_GARGLE,     IID_IDirectSoundFXGargle8,      @m_lpGargle);
    eSFX_parameq:    Result := EnableGenericFX(GUID_DSFX_STANDARD_PARAMEQ,    IID_IDirectSoundFXParamEq8,     @m_lpParamEq);
    eSFX_reverb:     Result := EnableGenericFX(GUID_DSFX_WAVES_REVERB,        IID_IDirectSoundFXWavesReverb8, @m_lpReverb);
  else
    Result := E_FAIL;
  end;
end;


//-----------------------------------------------------------------------------
// Name: DisableAllFX( )
// Desc: disables all effect in the DirectSoundBuffer, and releases all effect
//       object.
//-----------------------------------------------------------------------------
function CSoundFXManager.DisableAllFX: HRESULT;
begin
  // release all effect interfaces created with this manager so far
  SAFE_RELEASE(m_lpChorus);
  SAFE_RELEASE(m_lpCompressor);
  SAFE_RELEASE(m_lpDistortion);
  SAFE_RELEASE(m_lpEcho);
  SAFE_RELEASE(m_lpFlanger);
  SAFE_RELEASE(m_lpGargle);
  SAFE_RELEASE(m_lpParamEq);
  SAFE_RELEASE(m_lpReverb);

  m_dwNumFX := 0;
  ZeroMemory(@m_rgFxDesc, SizeOf(TDSEffectDesc) * eNUM_SFX);
  ZeroMemory(@m_rgRefGuids, SizeOf(PGUID) * eNUM_SFX);
  ZeroMemory(@m_rgPtrs, SizeOf(PPointer) * eNUM_SFX);
  ZeroMemory(@m_rgLoaded, SizeOf(BOOL) * eNUM_SFX);

  if (nil = m_lpDSB8) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  // Buffer must be stopped before calling SetFx
  if (m_lpDSB8 <> nil) then m_lpDSB8.Stop;

  // this removes all fx from the buffer
  m_lpDSB8.SetFX(0, nil, nil);

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: ActivateFX( )
// Desc: activate the effects enabled from EnableFX( )
//-----------------------------------------------------------------------------
function CSoundFXManager.ActivateFX: HRESULT;
var
  dwResults: array[0..eNUM_SFX-1] of DWORD;
  i: Integer;
begin
  if (nil = m_lpDSB8) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  if (m_dwNumFX = 0) then
  begin
    Result:= S_FALSE;
    Exit;
  end;

  Result := m_lpDSB8.SetFX(m_dwNumFX, @m_rgFxDesc, @dwResults);
  if FAILED(Result) then Exit;

  // get reference to the effect object
  for i := 0 to m_dwNumFX - 1 do
  begin
    Result := m_lpDSB8.GetObjectInPath(m_rgFxDesc[i].guidDSFXClass, 0, m_rgRefGuids[i]^, m_rgPtrs[i]^);
    if FAILED(Result) then
    begin
      // DXTrace(__FILE__, __LINE__, Result, 'GetObjectInPath', True);
      Result:= DXTrace(UnitName, $FFFFFFFF, Result, 'GetObjectInPath', True);
      Exit;
    end;
  end;

  Result:= S_OK;
end;


//-----------------------------------------------------------------------------
// Name: EnableGenericFX( )
// Desc: given information, tries to enabled an effect in a DirectSoundBuffer8,
//       and tries obtain reference to effect interface
//-----------------------------------------------------------------------------
function CSoundFXManager.EnableGenericFX(const guidSFXClass: TGUID;
  const rguidInterface: TGUID; ppObj: PPointer): HRESULT;
begin
  // if an effect already allocated
  if (ppObj^ <> nil) then
  begin
    Result:= S_FALSE;
    Exit;
  end;

  if (m_dwNumFX >= eNUM_SFX) then
  begin
    Result:= E_FAIL;
    Exit;
  end;

  // set the effect to be enabled
  ZeroMemory(@m_rgFxDesc[m_dwNumFX], SizeOf(TDSEffectDesc));
  m_rgFxDesc[m_dwNumFX].dwSize  := SizeOf(TDSEffectDesc);
  m_rgFxDesc[m_dwNumFX].dwFlags := 0;
  CopyMemory(@m_rgFxDesc[m_dwNumFX].guidDSFXClass, @guidSFXClass, SizeOf(TGUID));

  m_rgRefGuids[m_dwNumFX] := @rguidInterface;
  m_rgPtrs[m_dwNumFX] := ppObj;

  Inc(m_dwNumFX);

  Result:= S_OK;
end;

end.
